<?php

/* Carbon Fields Blocks */

use Carbon_Fields\Container;
use Carbon_Fields\Block;
use Carbon_Fields\Field;


add_action( 'carbon_fields_register_fields', 'crb_attach_standard_posttype_fields' );
function crb_attach_standard_posttype_fields() {
    
    /* Custom Post Type Registration: Alerts */
                /* Taxonomy for Alert Categories */
                $args = array(
                    "label"        => __( "Alert Categories", "pmtheme" ),
                    "public"       => true,
                    "rewrite"      => false,
                    "show_in_rest" => true,
                    "hierarchical" => true
                );
                
                register_taxonomy( "alerts_alert_categories", "alerts", $args );

                    
    register_post_type( 'alerts',
    // CPT Options
        array(
            'labels' => array(
                'name' => __( 'Alerts' ),
                'singular_name' => __( 'Alert' )
            ),
            'menu_icon' => 'dashicons-warning',
            'supports' => array( 'title', 'editor', 'thumbnail', 'revisions', 'custom-fields', 'page-attributes' ),
            'public' => true,
            'has_archive' => false,            'hierarchical' => false,            'rewrite' => array('slug' => 'alerts'),
            'show_in_rest' => true,
            'taxonomies' => array('alerts_alert_categories'),
            "template" => array(
                array(
                    "carbon-fields/alert"
                )
            )
  
        )
    );

        /* Custom Fields for Alerts */
    
    Container::make( 'post_meta', 'Alert Data' )
    ->where( 'post_type', '=', 'alerts' )
    ->add_fields( array(
        
                    ));

    /* Shortcode for Alert Listing */

    function display_alerts_listing_item($post_ID){

        $item_output = apply_filters("the_content", get_post_field("post_content", $post_ID));
                        return $item_output;
    }

    function display_alerts_listing($atts){ 

        $atts = shortcode_atts( array(
            'id' => '',
            'multiple' => false,
            'limit' => '-1',
                    ), $atts, 'display_alerts_listing' );

        $limit = $atts['limit'];

        if(!$atts['multiple']){

            $count = '';

            $post_ID = $atts['id'];

            $post_exists = (new WP_Query(['post_type' => 'any', 'p'=>$post_ID]))->found_posts > 0;

            if($post_exists){
                                $item_output = display_alerts_listing_item($post_ID);
            } else {
                $item_output = 'No Alerts Found.';
            }

        } else {

            $tax_query = array();

            $item_output = '';

            
            if(!empty($tax_query)){
                $posts = get_posts([
                    "post_type" => "alerts",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC",
                    "tax_query" => array( $tax_query )
                ]);
            } else {
                $posts = get_posts([
                    "post_type" => "alerts",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC"
                ]);
            }

            foreach($posts as $count => $current_post){
                    $post_ID = $current_post->ID;          

                $post_exists = (new WP_Query(["post_type" => "any", "p"=>$post_ID]))->found_posts > 0;

                if($post_exists){
                                        $item_output = display_alerts_listing_item($post_ID);
                } else {
                    $item_output = 'No Alerts Found.';
                }
            }


            return $item_output;
        }

    }

    add_shortcode('alerts_listing','display_alerts_listing');

    /* [alerts_listing id="" multiple=""] */

        /* Custom Block: Alert Listing Item */
    Block::make( __( 'Alert Display' ) )
        ->add_fields( array(
        Field::make( 'html', 'block_title' )->set_html( '<h2>Alert</h2>' ),
        
        Field::make( 'html', 'item_block_title' )->set_html( '<h3>Alert</h3>' ),
        Field::make( 'association', 'items_alerts', __( 'Alert Items' ) )->set_types(
            array( 
                array(
                    'type' => 'post',
                    'post_type' => 'alerts',
                )
            ) 
        )
        ) )->set_category( 'pannos-blocks', __( 'Pannos Template Blocks' ), 'schedule' )
    ->set_icon( 'warning' )
    ->set_render_callback( function ($fields, $attributes, $inner_blocks) {

            $block_uid = uniqid(); 
            if(isset($attributes['className'])) echo '<div class="'.$attributes['className'].'">';
?>
            <?php if(!empty($fields["items_alerts"])){
                    
                                            $alerts_list = array();
                    
                                            foreach($fields["items_alerts"] as $sc_count => $alerts_item){
                                                $alerts_list[$sc_count] = $alerts_item["id"];
                                            }   
                    
                                            // The Query
                    
                                            $sub_query = new WP_Query( array( 'post_type' => 'alerts', 'post__in' => $alerts_list, 'orderby' => 'post__in' ) );
                                            $counter = '0';
                                            // The Loop
                                            while ( $sub_query->have_posts() ) : $sub_query->the_post();
                                            
                                            echo display_alerts_listing_item(get_the_ID());
                                            
                                            endwhile;
                    
                                            // Reset Post Data
                                            wp_reset_postdata();
                                        }
                                        ?>
<?php   if(isset($attributes['className'])) echo '</div>';
} );
    /* End Custom Block Alerts */


    
    /* End Alerts Custom Post Type */

    /* Custom Post Type Registration: Feature */
                /* Taxonomy for Feature Categories */
                $args = array(
                    "label"        => __( "Feature Categories", "pmtheme" ),
                    "public"       => true,
                    "rewrite"      => false,
                    "show_in_rest" => true,
                    "hierarchical" => true
                );
                
                register_taxonomy( "feature_categories", "feature", $args );

                    
    register_post_type( 'feature',
    // CPT Options
        array(
            'labels' => array(
                'name' => __( 'Feature' ),
                'singular_name' => __( 'Featured Item' )
            ),
            'menu_icon' => 'dashicons-slides',
            'supports' => array( 'title', 'editor', 'thumbnail', 'revisions', 'custom-fields', 'page-attributes' ),
            'public' => true,
            'has_archive' => false,            'hierarchical' => false,            'rewrite' => array('slug' => 'feature'),
            'show_in_rest' => true,
            
            "template" => array(
                array(
                    "carbon-fields/feature-slide"
                )
            ),            'taxonomies' => array('feature_categories')
  
        )
    );

        /* Custom Fields for Feature */
    
    Container::make( 'post_meta', 'Featured Item Data' )
    ->where( 'post_type', '=', 'feature' )
    ->add_fields( array(

                
                    ));

    /* Shortcode for Featured Item Listing */

    function display_feature_listing($atts){ 

        $atts = shortcode_atts( array(
            'id' => '',
            'multiple' => false,
            'limit' => '-1',
            'feature_categories' => '',
                            ), $atts, 'display_feature_listing' );

        $limit = $atts['limit'];

        if(!$atts['multiple']){

            $count = '';

            $post_ID = $atts['id'];

            $post_exists = (new WP_Query(['post_type' => 'any', 'p'=>$post_ID]))->found_posts > 0;

            if($post_exists){
                $item_output = '<div class="js-pannosFeature-slide pannosFeature-slide js-pannosFeature-slide'.$count.'">'.apply_filters("the_content", get_post_field("post_content", $post_ID)).'</div>';         } else {
                $item_output = 'No slides to display.';
            }

        } else {

            $tax_query = array();

            $item_output = '';

            

                        if(!empty($atts['feature_categories'])){

                            $post_terms = explode(",",$atts['feature_categories']);

                            $tax_query['feature_categories'] = 
                                array(
                                    "taxonomy" => 'feature_categories',
                                    "field"    => "slug",
                                    "terms"    => $post_terms
                                );

                        }
                    
            if(!empty($tax_query)){
                $posts = get_posts([
                    "post_type" => "feature",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC",
                    "tax_query" => $tax_query
                ]);
            } else {
                $posts = get_posts([
                    "post_type" => "feature",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC"
                ]);
            }

            $item_output = '<section aria-label="Featured Content" id="sec-feature" class="home-feature">';

            foreach($posts as $count => $current_post){
                    $post_ID = $current_post->ID;          

                $post_exists = (new WP_Query(["post_type" => "any", "p"=>$post_ID]))->found_posts > 0;

                if($post_exists){

$item_output .= '<div class="js-pannosFeature-slide pannosFeature-slide js-pannosFeature-slide'.$count.'">'.apply_filters("the_content", get_post_field("post_content", $post_ID)).'</div>';                } else {
                    $item_output = 'No slides to display.';
                }
            }

            $item_output .= '</section>';

            return $item_output;
        }

    }

    add_shortcode('feature_listing','display_feature_listing');

    /* [feature_listing id="" multiple=""] */



    /* Shortcode for Featured Item Slider */

    function display_feature_slider($atts){ 
        $atts = shortcode_atts( array(
            'limit' => '-1',
            'feature_categories' => '',
            'aria_label' => 'Feature Slide',
            'mobile_static' => false,
                            ), $atts, 'display_feature_slider' );

        $limit = $atts['limit'];

            $tax_query = array();

            $item_output = '';

            $custom_class = '';

            if($atts['mobile_static'] && $atts['mobile_static'] != "0" && $atts['mobile_static'] != "false"){
                $data_options = 'data-splide=\'{"breakpoints":{"950":{"autoplay":"pause","destroy":true,"autoHeight":true,"drag":false}}}\'';
                $custom_class = ' static_slide';
            } else $data_options = 'data-splide=\'{"breakpoints":{"950":{"autoplay":"pause","destroy":false,"autoHeight":true,"drag":true}}}\'';

            

                        if(!empty($atts['feature_categories'])){

                            $post_terms = explode(",",$atts['feature_categories']);

                            $tax_query['feature_categories'] = 
                                array(
                                    "taxonomy" => 'feature_categories',
                                    "field"    => "slug",
                                    "terms"    => $post_terms
                                );

                        }
                    
            if(!empty($tax_query)){
                $posts = get_posts([
                    "post_type" => "feature",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC",
                    "tax_query" => $tax_query
                ]);
            } else {
                $posts = get_posts([
                    "post_type" => "feature",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC"
                ]);
            }

            $item_output = '<section class="splide mod-feature home-feature is-overflow'.$custom_class.'" '.$data_options.'aria-label="'.$atts['aria_label'].'">
                  <div class="splide__track">
                    <ul class="splide__list">';

            foreach($posts as $count => $current_post){
                    $post_ID = $current_post->ID;          

                $post_exists = (new WP_Query(["post_type" => "any", "p"=>$post_ID]))->found_posts > 0;

                if($post_exists){

$item_output .= '
    <li class="splide__slide">
        <div class="splide__slide__container">
            <div class="pannosFeature-slide js-pannosFeature-slide'.$count.'">
            '.apply_filters("the_content", get_post_field("post_content", $post_ID)).'
            </div>
        </div>
    </li>';                } else {
                    $item_output = '';
                }
            }

            
            if(isset(carbon_get_theme_option('feature_subcontent')[0]['id']) && (!empty(carbon_get_theme_option('feature_subcontent')[0]['id']))) {
                $subcontentShortcode = do_shortcode('[subcontent_item id="'.carbon_get_theme_option('feature_subcontent')[0]['id'].'"]');
            } else {
                $subcontentShortcode = "";
            };

            $item_output .= '
                    </ul>
                  </div>
                  <div class="js-pannosFeature-controls pannosFeature-controls">
                      <button class="splide__toggle" type="button">
                          <span class="splide__toggle__play pannosFeature-pause js-paused"><span class="visuallyhidden">Play</span></span>
                          <span class="splide__toggle__pause pannosFeature-pause"><span class="visuallyhidden">Pause</span></span>
                      </button>
                      <ul class="splide__pagination pannosFeature-dots"></ul>
                  </div>'.$subcontentShortcode.'
                </section>';

            return $item_output;

    }

    add_shortcode('feature_slider','display_feature_slider');

    /* [feature_slider mobile_static="0"] */

    
    /* End Feature Custom Post Type */

    /* Custom Post Type Registration: Subcontent */
                /* Taxonomy for Subcontent Categories */
                $args = array(
                    "label"        => __( "Subcontent Categories", "pmtheme" ),
                    "public"       => true,
                    "rewrite"      => false,
                    "show_in_rest" => true,
                    "hierarchical" => true,
                    "show_admin_column" => true
                );
                
                register_taxonomy( "subcontent_categories", "subcontent", $args );

                    
    register_post_type( 'subcontent',
    // CPT Options
        array(
            'labels' => array(
                'name' => __( 'Subcontent' ),
                'singular_name' => __( 'Subcontent' )
            ),
            'menu_icon' => 'dashicons-images-alt',
            'supports' => array( 'title', 'editor', 'thumbnail', 'revisions', 'custom-fields', 'page-attributes' ),
            'public' => true,
            'has_archive' => true,            'hierarchical' => false,            'rewrite' => array('slug' => 'subcontent'),
            'show_in_rest' => true,
                        'taxonomies' => array('subcontent_categories')
  
        )
    );

    Container::make( 'post_meta', 'Shortcode Usage:' )
        ->where( 'post_type', '=', 'subcontent' )->set_priority( 'high' )
        ->add_fields( array(
            Field::make( 'html', 'shortcode_info' )
            ->set_html( 
                function(){
                    $post_id = $_GET['post'] ?? $_POST['id'] ?? $_POST['post_ID']; 
                    $slug = get_post_field( 'post_name', $post_id );
                    return '
                    <p>Use the subcontent_item shortcode with the post ID:</p>
                    <p><strong>[subcontent_item id="'.$post_id.'"]</strong></p>
                    <p>Or use the global_tag shortcode with the permalink as "code" value:</p>
                    <p><strong>[global_tag code="'.$slug.'"]</strong></p>
                    <p>In a php file you can also output this using the function global_tag('.$slug.') with the permalink.</p>
                    <p>To output a global tag as a plain text value instead of the post content, check the box and fill the field below.</p>
                    ';
                } ),
            Field::make( 'checkbox', 'text_output', __( 'Output Text Instead' ) )->set_option_value( '1' )->set_default_value( '0' ),
            Field::make( 'text', 'text_output_value', __( 'Text to Output' ) )->set_conditional_logic( array(
                array(
                    'field' => 'text_output',
                    'value' => '1',
                    'compare' => '='
                )
            ) )
        ));

    /* Simulate Global Tag */   

    function global_tag($post_name){

        $subcontent = get_page_by_path($post_name, OBJECT, 'subcontent');

        if(carbon_get_post_meta( $subcontent->ID, 'text_output' ) && carbon_get_post_meta( $subcontent->ID, 'text_output' ) == 1){
            if(carbon_get_post_meta( $subcontent->ID, 'text_output_value' ) && !empty(carbon_get_post_meta( $subcontent->ID, 'text_output_value' ))){
                return carbon_get_post_meta( $subcontent->ID, 'text_output_value' );
            }
        }

        $item_output = apply_filters("the_content", get_post_field("post_content", $subcontent->ID));
        
        return $item_output;
    }

    function display_global_tag($atts){ 

        $atts = shortcode_atts( array(
            'code' => '',
                            ), $atts, 'display_global_tag' );

        $code = $atts['code'];
        if(!empty($code)){
            return global_tag($code);
        }        
        return '<!-- Subcontent Not Found -->';
    }
    add_shortcode('global_tag','display_global_tag');
    /* [global_tag code=""] */
    
    /* Shortcode for Subcontent Listing */

    function display_subcontent_listing_item($post_ID){

        $item_output = apply_filters("the_content", get_post_field("post_content", $post_ID));
                        return $item_output;
    }

    function display_subcontent_listing($atts){ 

        $atts = shortcode_atts( array(
            'id' => '',
            'multiple' => false,
            'limit' => '-1',
            'subcontent_categories' => '',
                            ), $atts, 'display_subcontent_listing' );

        $limit = $atts['limit'];

        if(!$atts['multiple']){

            $count = '';

            $post_ID = $atts['id'];

            $post_exists = (new WP_Query(['post_type' => 'any', 'p'=>$post_ID]))->found_posts > 0;

            if($post_exists){
                                $item_output = display_subcontent_listing_item($post_ID);
            } else {
                $item_output = 'No Subcontent Found.';
            }

        } else {

            $tax_query = array();

            $item_output = '';

            

                        if(!empty($atts['subcontent_categories'])){

                            $post_terms = explode(",",$atts['subcontent_categories']);

                            $tax_query['subcontent_categories'] = 
                                array(
                                    "taxonomy" => 'subcontent_categories',
                                    "field"    => "slug",
                                    "terms"    => $post_terms
                                );

                        }
                    
            if(!empty($tax_query)){
                $posts = get_posts([
                    "post_type" => "subcontent",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC",
                    "tax_query" => $tax_query
                ]);
            } else {
                $posts = get_posts([
                    "post_type" => "subcontent",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC"
                ]);
            }

            foreach($posts as $count => $current_post){
                    $post_ID = $current_post->ID;          

                $post_exists = (new WP_Query(["post_type" => "any", "p"=>$post_ID]))->found_posts > 0;

                if($post_exists){
                                        $item_output = display_subcontent_listing_item($post_ID);
                } else {
                    $item_output = 'No Subcontent Found.';
                }
            }


            return $item_output;
        }

    }

    add_shortcode('subcontent_listing','display_subcontent_listing');

    /* [subcontent_listing id="" multiple=""] */

    function display_subcontent_item($atts){ 

        $atts = shortcode_atts( array(
            'id' => '',
                            ), $atts, 'display_subcontent_item' );

        $id = $atts['id'];
        if(!empty($id)){
            return display_subcontent_listing_item($id);
        }
        return '<!-- Subcontent Not Found -->';
    }
    add_shortcode('subcontent_item','display_subcontent_item');

    /* [subcontent_item id=""] */

        /* Custom Block: Subcontent Listing Item */
    Block::make( __( 'Subcontent' ) )
        ->add_fields( array(
        Field::make( 'html', 'block_title' )->set_html( '<h2>Subcontent</h2>' ),
        
        Field::make( 'html', 'item_block_title' )->set_html( '<h3>Subcontent</h3>' ),
        Field::make( 'association', 'items_subcontent', __( 'Subcontent Items' ) )->set_types(
            array( 
                array(
                    'type' => 'post',
                    'post_type' => 'subcontent',
                )
            ) 
        )
        ) )->set_category( 'pannos-blocks', __( 'Pannos Template Blocks' ), 'schedule' )
    ->set_icon( 'images-alt' )
    ->set_render_callback( function ($fields, $attributes, $inner_blocks) {

            $block_uid = uniqid(); 
            if(isset($attributes['className'])) echo '<div class="'.$attributes['className'].'">';
?>
            <?php if(!empty($fields["items_subcontent"])){
                    
                                            $subcontent_list = array();
                    
                                            foreach($fields["items_subcontent"] as $sc_count => $subcontent_item){
                                                $subcontent_list[$sc_count] = $subcontent_item["id"];
                                            }   
                    
                                            // The Query
                    
                                            $sub_query = new WP_Query( array( 'post_type' => 'subcontent', 'post__in' => $subcontent_list, 'orderby' => 'post__in' ) );
                                            $counter = '0';
                                            // The Loop
                                            while ( $sub_query->have_posts() ) : $sub_query->the_post();
                                            
                                            echo display_subcontent_listing_item(get_the_ID());
                                            
                                            endwhile;
                    
                                            // Reset Post Data
                                            wp_reset_postdata();
                                        }
                                        ?>
<?php   if(isset($attributes['className'])) echo '</div>';
} );
    /* End Custom Block Subcontent */

    $post_type = 'subcontent';
    // Register the columns.
    add_filter( "manage_{$post_type}_posts_columns", function ( $defaults ) {
    $defaults['subcontent_shortcode'] = 'Subcontent Shortcode';
    return $defaults;
    } );
    // Handle the value for each of the new columns.
    add_action( "manage_{$post_type}_posts_custom_column", function ( $column_name, $post_id ) {
    if ( $column_name == 'subcontent_shortcode' ) {
    // Display an ACF field
    echo '<strong>[subcontent_item id="'.$post_id.'"]</strong>';
    }
    }, 10, 2 );

    /* End Subcontent Custom Post Type */


    



    /* Custom Block: Rate Table */
Block::make( __( 'Rate Table' ) )
    ->add_fields( array(
    Field::make( 'html', 'block_title' )->set_html( '<h2>Rate Table</h2>' ),
    
    Field::make( 'html', 't863130_block_title' )->set_html( '<h3>Rate Table</h3>' ),
    Field::make( 'association', 't863130_pmrates_rate', __( 'Rate Table  Items' ) )->set_types(
        array( 
            array(
                'type' => 'post',
                'post_type' => 'pmrates_rate',
            )
        ) 
    ),
    Field::make( 'checkbox', 't863130_showtitle', __( 'Show Table Title' ) )->set_option_value( '1' )->set_width( 24 ),
    Field::make( 'checkbox', 't863130_showdate', __( 'Show Effective Date' ) )->set_option_value( '1' )->set_width( 24 )->set_default_value( '1' ),
    Field::make( 'checkbox', 't863130_showdisclosure', __( 'Show Disclosure' ) )->set_option_value( '1' )->set_width( 24 )->set_default_value( '1' ),
    Field::make( 'checkbox', 't863130_showprint', __( 'Show Print Table Button' ) )->set_option_value( '1' )->set_width( 24 ),      
    Field::make( 'checkbox', 't863130_showall', __( 'Show All Columns and Rows' ) )->set_option_value( '1' )->set_width( 24 )->set_default_value( '1' ),
    Field::make( 'text', 't863130_hiderows', __( 'Hide Rows (comma separated)' ) )->set_width( 29 )->set_conditional_logic( array(
                    array(
                        'field' => 't863130_showall',
                        'value' => '0',
                        'compare' => '='
                    )
                ) ),
    Field::make( 'text', 't863130_hidecolumns', __( 'Hide Columns (comma separated)' ) )->set_width( 29 )->set_conditional_logic( array(
                    array(
                        'field' => 't863130_showall',
                        'value' => '0',
                        'compare' => '='
                    )
                ) )
    ) )->set_category( 'pannos-blocks', __( 'Pannos Template Blocks' ), 'schedule' )
->set_icon( 'chart-line' )
->set_render_callback( function ($fields, $attributes, $inner_blocks) {

        $block_uid = uniqid();

     
            if(isset($attributes['className'])) echo '<div class="'.$attributes['className'].'">';
?>
            
                    <?php if(!empty($fields["t863130_pmrates_rate"])){

                        $pmrates_rate_list = array();

                        foreach($fields["t863130_pmrates_rate"] as $sc_count => $pmrates_rate_item){
                            $pmrates_rate_list[$sc_count] = $pmrates_rate_item["id"];
                        }   

                        // The Query

                        $sub_query = new WP_Query( array( 'post_type' => 'pmrates_rate', 'post__in' => $pmrates_rate_list, 'orderby' => 'post__in' ) );
                        $counter = '0';

                        // The Loop
                        while ( $sub_query->have_posts() ) : $sub_query->the_post();
                        
                        if ( class_exists('RateTable') ) { 
                            $rateTable = new RateTable();
                            if(isset($fields['t863130_showtitle'])) $t863130_showtitle = $fields['t863130_showtitle']; else $t863130_showtitle = 0;
                            if(isset($fields['t863130_showdate'])) $t863130_showdate = $fields['t863130_showdate']; else $t863130_showdate = 0;
                            if(isset($fields['t863130_showdisclosure'])) $t863130_showdisclosure = $fields['t863130_showdisclosure']; else $t863130_showdisclosure = 0;
                            if(isset($fields['t863130_showprint'])) $t863130_showprint = $fields['t863130_showprint']; else $t863130_showprint = 0;
                            if(isset($fields['t863130_showall'])) $t863130_showall = $fields['t863130_showall']; else $t863130_showall = 0;
                            if(isset($fields['t863130_hiderows'])) $t863130_hiderows = $fields['t863130_hiderows']; else $t863130_hiderows = '';
                            if(isset($fields['t863130_hidecolumns'])) $t863130_hidecolumns = $fields['t863130_hidecolumns']; else $t863130_hidecolumns = '';
                            $rateTableOutput = $rateTable->render( 
                            array('rateTableID' => get_the_ID(), 
                            'rateTableAnchor' => 'ratetable_'.get_the_ID(), 
                            'rateTableMargin' => 'margin-bottom-large',
                            'showtitle' => $t863130_showtitle,
                            'showdate' => $t863130_showdate,
                            'showdisclosure' => $t863130_showdisclosure,
                            'showprint' => $t863130_showprint,
                            'showall' => $t863130_showall,
                            'hiderows' => $t863130_hiderows,
                            'hidecolumns' => $t863130_hidecolumns
                            ));

                            echo $rateTableOutput; 
                        }
                        
                        $counter++;

                        endwhile;

                        // if($counter == '0') echo 'No Posts.';

                        // Reset Post Data
                        wp_reset_postdata();
                    }
                    ?>
                    
<?php   if(isset($attributes['className'])) echo '</div>';
} );
    /* End Custom Block Rate Table */

/* Custom Post Type Registration: Products */
        /* Taxonomy for Product Categories */
        $args = array(
            "label"        => __( "Product Categories", "pmtheme" ),
            "public"       => true,
            "rewrite"      => false,
            "show_in_rest" => true,
            "hierarchical" => true
        );
        
        register_taxonomy( "product_categories", "products", $args );
    
    register_post_type( 'product',
    // CPT Options
        array(
            'labels' => array(
                'name' => __( 'Products' ),
                'singular_name' => __( 'Product' )
            ),
            'menu_icon' => 'dashicons-products',
            'supports' => array( 'title', 'editor', 'thumbnail', 'revisions', 'custom-fields', 'page-attributes' ),
            'public' => true,
            'has_archive' => true,            'hierarchical' => false,            'rewrite' => array('slug' => 'product'),
            'show_in_rest' => true,
                        'taxonomies' => array('product_categories')
  
        )
    );

    /* Shortcode for Product Listing */

    function display_product_listing($atts){ 

        $atts = shortcode_atts( array(
            'id' => '',
            'multiple' => false,
            'limit' => '-1',
                    ), $atts, 'display_product_listing' );

        $limit = $atts['limit'];

        if(!$atts['multiple']){

            $count = '';

            $post_ID = $atts['id'];

            $post_exists = (new WP_Query(['post_type' => 'any', 'p'=>$post_ID]))->found_posts > 0;

            if($post_exists){
                $item_output = '';          } else {
                $item_output = 'No Products Found.';
            }

        } else {

            $tax_query = array();

            $item_output = '';

            
            if(!empty($tax_query)){
                $posts = get_posts([
                    "post_type" => "product",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC",
                    "tax_query" => $tax_query
                ]);
            } else {
                $posts = get_posts([
                    "post_type" => "product",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC"
                ]);
            }

            foreach($posts as $count => $current_post){
                    $post_ID = $current_post->ID;          

                $post_exists = (new WP_Query(["post_type" => "any", "p"=>$post_ID]))->found_posts > 0;

                if($post_exists){

$item_output .= '';                } else {
                    $item_output = 'No Products Found.';
                }
            }


            return $item_output;
        }

    }

    add_shortcode('product_listing','display_product_listing');

    /* [product_listing id="" multiple=""] */



    /* Custom Block: Product Selector */
Block::make( __( 'Product Selector' ) )
    ->add_fields( array(
    Field::make( 'html', 'block_title' )->set_html( '<h2>Product Selector</h2>' ),
    
    Field::make( 'html', 't493254_block_title' )->set_html( '<h3>Product(s)</h3>' ),
    Field::make( 'association', 't493254_products', __( 'Products  Items' ) )->set_types(
        array( 
            array(
                'type' => 'post',
                'post_type' => 'product',
            )
        ) 
    )
    ) )->set_category( 'pannos-blocks', __( 'Pannos Template Blocks' ), 'schedule' )
->set_icon( 'excerpt-view' )
->set_render_callback( function ($fields, $attributes, $inner_blocks) {

        $block_uid = uniqid();

     
            if(isset($attributes['className'])) echo '<div class="'.$attributes['className'].'">';
?>
            
                    <?php if(!empty($fields["t493254_products"])){

                        $products_list = array();

                        foreach($fields["t493254_products"] as $sc_count => $products_item){
                            $products_list[$sc_count] = $products_item["id"];
                        }   

                        // The Query

                        $sub_query = new WP_Query( array( 'post_type' => 'product', 'post__in' => $products_list, 'orderby' => 'post__in' ) );
                        $counter = '0';

                        // The Loop
                        while ( $sub_query->have_posts() ) : $sub_query->the_post();
                        
                        printf(the_content());
                        
                        $counter++;

                        endwhile;

                        // if($counter == '0') echo 'No Posts.';

                        // Reset Post Data
                        wp_reset_postdata();
                    }
                    ?>
                    
<?php   if(isset($attributes['className'])) echo '</div>';
} );
    /* End Custom Block Product Selector */
    
    
    /* End Products Custom Post Type */
    


    /* Custom Post Type Registration: Links */  
    /* Taxonomy for Old Site ID */
    $args = array(
        "label"        => __( "Old Site ID (Optional)", "pmtheme" ),
        "public"       => true,
        "rewrite"      => false,
        "show_in_rest" => true,
        "hierarchical" => false
    );
    
    register_taxonomy( "links_old_site_id", array( 'page', 'links' ), $args );

    register_post_type( 'links',
    // CPT Options
        array(
            'labels' => array(
                'name' => __( 'Links' ),
                'singular_name' => __( 'Link' )
            ),
            'menu_icon' => 'dashicons-admin-links',
            'supports' => array( 'title', 'revisions', 'custom-fields', 'page-attributes' ),
            'public' => true,
            'has_archive' => false,            'hierarchical' => false,            'rewrite' => array('slug' => 'links'),
            'show_in_rest' => true,
                        'taxonomies' => array()
  
        )
    );

        /* Custom Fields for Links */
    
    Container::make( 'post_meta', 'Link Data' )
    ->where( 'post_type', '=', 'links' )
    ->add_fields( array(
        Field::make( 'text', 'url', 'URL' ),
                
                Field::make( 'checkbox', 'target', 'Open in New Window' )->set_option_value( '_blank' ),
                
                Field::make( 'checkbox', 'class', 'Add Speedbump' )->set_option_value( 'speedbump' ),
                
                Field::make( 'text', 'title', 'Title' ),
                
                    ));

    
    /* Shortcode for Link Listing */

    function display_links_markup_item($post_ID){

        
$item_output = '<a href="'.carbon_get_post_meta( $post_ID, 'url' ).'" target="'.carbon_get_post_meta( $post_ID, 'target' ).'" class="'.carbon_get_post_meta( $post_ID, 'class' ).'">'.carbon_get_post_meta( $post_ID, 'title' ).'</a>';
                
return $item_output;
    }

    function display_links_markup($atts){ 

        $atts = shortcode_atts( array(
            'id' => '',
            'multiple' => false,
            'limit' => '-1',
                    ), $atts, 'display_links_markup' );

        $limit = $atts['limit'];

        if(!$atts['multiple']){

            $count = '';

            $post_ID = $atts['id'];

            $post_exists = (new WP_Query(['post_type' => 'any', 'p'=>$post_ID]))->found_posts > 0;

            if($post_exists){
                                $item_output = display_links_markup_item($post_ID);
            } else {
                $item_output = 'No Links Found.';
            }

        } else {

            $tax_query = array();

            $item_output = '';

            
            if(!empty($tax_query)){
                $posts = get_posts([
                    "post_type" => "links",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC",
                    "tax_query" => array( $tax_query )
                ]);
            } else {
                $posts = get_posts([
                    "post_type" => "links",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC"
                ]);
            }

            foreach($posts as $count => $current_post){
                    $post_ID = $current_post->ID;          

                $post_exists = (new WP_Query(["post_type" => "any", "p"=>$post_ID]))->found_posts > 0;

                if($post_exists){
                                        $item_output = display_links_markup_item($post_ID);
                } else {
                    $item_output = 'No Links Found.';
                }
            }


            return $item_output;
        }

    }

    add_shortcode('links_markup','display_links_markup');

    /* [links_markup id="" multiple=""] */

    
        /* Shortcode for Link URL */

        function display_links_url_item($post_ID){
            $item_output = ''.carbon_get_post_meta( $post_ID, 'url' ).'';
            return $item_output;        }

        function display_links_url($atts){ 

            $atts = shortcode_atts( array(
                'id' => '',
                'multiple' => false,
                'limit' => '-1',
                            ), $atts, 'display_links_url' );

            $limit = $atts['limit'];

            if(!$atts['multiple']){

                $post_ID = $atts['id'];

                $post_exists = (new WP_Query(['post_type' => 'any', 'p'=>$post_ID]))->found_posts > 0;

                if($post_exists){
                                        $item_output = display_links_url_item($post_ID);
                } else {
                    $item_output = 'No Links Found.';
                }

            } else {

                $tax_query = array();

                $item_output = '';

                
                if(!empty($tax_query)){
                    $posts = get_posts([
                        "post_type" => "links",
                        "post_status" => "publish",
                        "numberposts" => $limit,
                        "order"    => "ASC",
                        "tax_query" => array( $tax_query )
                    ]);
                } else {
                    $posts = get_posts([
                        "post_type" => "links",
                        "post_status" => "publish",
                        "numberposts" => $limit,
                        "order"    => "ASC"
                    ]);
                }

                foreach($posts as $count => $current_post){
                    $post_ID = $current_post->ID;               

                    $post_exists = (new WP_Query(["post_type" => "any", "p"=>$post_ID]))->found_posts > 0;

                    if($post_exists){
                                                $item_output = display_links_url_item($post_ID);
                    } else {
                        $item_output = 'No Links Found.';
                    }
                }


                return $item_output;
            }

        }

        add_shortcode('links_url','display_links_url');

        /* [links_url id="" multiple=""] */
        /* Custom Block: Link Listing Item */
    Block::make( __( 'Link' ) )
        ->add_fields( array(
        Field::make( 'html', 'block_title' )->set_html( '<h2>Link</h2>' ),
        
        Field::make( 'html', 'item_block_title' )->set_html( '<h3>Link</h3>' ),
        Field::make( 'association', 'items_links', __( 'Link Items' ) )->set_types(
            array( 
                array(
                    'type' => 'post',
                    'post_type' => 'links',
                )
            ) 
        )
        ) )->set_category( 'pannos-blocks', __( 'Pannos Template Blocks' ), 'schedule' )
    ->set_icon( 'admin-links' )
    ->set_render_callback( function ($fields, $attributes, $inner_blocks) {

            $block_uid = uniqid(); 
            if(isset($attributes['className'])) echo '<div class="'.$attributes['className'].'">';
?>
            <?php if(!empty($fields["items_links"])){
                    
                                            $links_list = array();
                    
                                            foreach($fields["items_links"] as $sc_count => $links_item){
                                                $links_list[$sc_count] = $links_item["id"];
                                            }   
                    
                                            
                    
                                            $sub_query = new WP_Query( array( 'post_type' => 'links', 'post__in' => $links_list, 'orderby' => 'post__in' ) );
                                            $counter = '0';
                                            
                                            while ( $sub_query->have_posts() ) : $sub_query->the_post();
                                            
                                            echo display_links_markup_item(get_the_ID());
                                            
                                            endwhile;
                    
                                            
                                            wp_reset_postdata();
                                        }
                                        ?>
<?php   if(isset($attributes['className'])) echo '</div>';
} );
    /* End Custom Block Links */

    Container::make( 'post_meta', 'Shortcode Usage:' )
        ->where( 'post_type', '=', 'links' )->set_priority( 'high' )
        ->add_fields( array(
            Field::make( 'html', 'link_shortcode_info' )
            ->set_html( 
                function(){
                    $post_id = $_GET['post'] ?? $_POST['id'] ?? $_POST['post_ID']; 
                    $slug = get_post_field( 'post_name', $post_id );
                    return '
                    <p>Use the link_markup shortcode with the post ID to get the full formatted link:</p>
                    <p><strong>[link_markup id="'.$post_id.'"]</strong></p>
                    <p>Use the link_url shortcode with the post ID to get the raw URL only:</p>
                    <p><strong>[link_url id="'.$post_id.'"]</strong></p>
                    <p>In a php file you can also output this using the function link_output('.$post_id.',<em>true</em>) with the post ID. The second argument can be "true" for the full html markup, or "false" for the raw URL only.</p>
                    ';
                } )
        ));


    /* Simulate Global Tag */   

    function link_output($link_id,$markup = false,$class=''){

        if(!$markup){
            return carbon_get_post_meta( $link_id, 'url' );
        }

        $item_output = display_links_markup_item($link_id);
        
        return $item_output;
    }

    function display_link_url($atts){ 

        $atts = shortcode_atts( array(
            'id' => '',
                            ), $atts, 'display_link_url' );

        $code = $atts['id'];
        if(!empty($code)){
            return link_output($code,false);
        }        
        return '<!-- Subcontent Not Found -->';
    }
    add_shortcode('link_url','display_link_url');
    /* [link_url code=""] */

    function display_link_markup($atts){ 

        $atts = shortcode_atts( array(
            'id' => '',
            'class' => '',
                            ), $atts, 'display_link_markup' );

        $code = $atts['id'];
        $class = $atts['class'];
        if(!empty($code)){
            return link_output($code,true,$class);
        }        
        return '<!-- Subcontent Not Found -->';
    }
    add_shortcode('link_markup','display_link_markup');
    /* [link_markup code=""] */

    $post_type = 'links';
    // Register the columns.
    add_filter( "manage_{$post_type}_posts_columns", function ( $defaults ) {
    $defaults['links_url'] = 'Link URL';
    $defaults['links_url_shortcode'] = 'Link URL Shortcode';
    return $defaults;
    } );
    // Handle the value for each of the new columns.
    add_action( "manage_{$post_type}_posts_custom_column", function ( $column_name, $post_id ) {
    if ( $column_name == 'links_url' ) {
    echo carbon_get_post_meta( $post_id, 'url' );
    } 
    else if ( $column_name == 'links_url_shortcode' ) {
    // Display an ACF field
    echo '<strong>[link_url id="'.$post_id.'"]</strong>';
    }
    }, 10, 2 );

    /* End Links Custom Post Type */
    

    /* Custom Post Type Registration: FAQs */
                /* Taxonomy for FAQ Categories */
                $args = array(
                    "label"        => __( "FAQ Categories", "pmtheme" ),
                    "public"       => true,
                    "rewrite"      => false,
                    "show_in_rest" => true,
                    "hierarchical" => true,
                    "show_admin_column" => true,
                );
                
                register_taxonomy( "faq_categories", "faq", $args );

                    
    register_post_type( 'faq',
    // CPT Options
        array(
            'labels' => array(
                'name' => __( 'FAQs' ),
                'singular_name' => __( 'FAQ' )
            ),
            'menu_icon' => 'dashicons-editor-ul',
            'supports' => array( 'title', 'revisions', 'custom-fields', 'page-attributes' ),
            'public' => true,
            'has_archive' => false,            'hierarchical' => false,            'rewrite' => array('slug' => 'faq'),
            'show_in_rest' => true,
            'taxonomies' => array('faq_categories')
  
        )
    );

        /* Custom Fields for FAQs */
    
    Container::make( 'post_meta', 'FAQ Data' )
    ->where( 'post_type', '=', 'faq' )
    ->add_fields( array(
        
                Field::make( 'text', 'question', 'Question' ),
                
                Field::make( 'rich_text', 'answer', 'Answer' ),
                
                    ));

    
    /* Shortcode for FAQ Listing */

    function display_faq_listing_item($post_ID){

        
$item_output = '<li><a name="question_'.$post_ID.'"></a><button class="question js-accordion-title" tabindex="0" aria-expanded="false" aria-controls="ac_'.$post_ID.'" id="ach_'.$post_ID.'"><p>'.carbon_get_post_meta($post_ID, "question").'</p></button><div class="answer js-accordion-content" id="ac_'.$post_ID.'" aria-labelledby="ach_'.$post_ID.'" role="region" style="display: none;">'.wpautop(carbon_get_post_meta($post_ID, "answer")).'</div></li>';
                
return $item_output;
    }

    function display_faq_listing($atts){ 

        $atts = shortcode_atts( array(
            'id' => '',
            'multiple' => false,
            'limit' => '-1',
            'faq_categories' => '',
                            ), $atts, 'display_faq_listing' );

        $limit = $atts['limit'];

        if(!$atts['multiple']){

            $count = '';

            $post_ID = $atts['id'];

            $post_exists = (new WP_Query(['post_type' => 'any', 'p'=>$post_ID]))->found_posts > 0;

            if($post_exists){
                                $item_output = display_faq_listing_item($post_ID);
            } else {
                $item_output = 'No FAQs Found.';
            }

        } else {

            $tax_query = array();

            $item_output = '';

            

                        if(!empty($atts['faq_categories'])){

                            $post_terms = explode(",",$atts['faq_categories']);

                            $tax_query['faq_categories'] = 
                                array(
                                    "taxonomy" => 'faq_categories',
                                    "field"    => "slug",
                                    "terms"    => $post_terms
                                );

                        }
                    
            if(!empty($tax_query)){
                $posts = get_posts([
                    "post_type" => "faq",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC",
                    "tax_query" => array( $tax_query )
                ]);
            } else {
                $posts = get_posts([
                    "post_type" => "faq",
                    "post_status" => "publish",
                    "numberposts" => $limit,
                    "order"    => "ASC"
                ]);
            }

            foreach($posts as $count => $current_post){
                    $post_ID = $current_post->ID;          

                $post_exists = (new WP_Query(["post_type" => "any", "p"=>$post_ID]))->found_posts > 0;

                if($post_exists){
                                        $item_output = display_faq_listing_item($post_ID);
                } else {
                    $item_output = 'No FAQs Found.';
                }
            }


            return $item_output;
        }

    }

    add_shortcode('faq_listing','display_faq_listing');

    /* [faq_listing id="" multiple=""] */

        /* Custom Block: FAQ Listing Item */
    Block::make( __( 'FAQ' ) )
        ->add_fields( array(
        Field::make( 'html', 'block_title' )->set_html( '<h2>FAQ</h2>' ),
        
        Field::make( 'html', 'item_block_title' )->set_html( '<h3>FAQ</h3>' ),
        Field::make( 'association', 'items_faq', __( 'FAQ Items' ) )->set_types(
            array( 
                array(
                    'type' => 'post',
                    'post_type' => 'faq',
                )
            ) 
        )
        ) )->set_category( 'pannos-blocks', __( 'Pannos Template Blocks' ), 'schedule' )
    ->set_icon( 'editor-ul' )
    ->set_render_callback( function ($fields, $attributes, $inner_blocks) {

            $block_uid = uniqid(); 
            if(isset($attributes['className'])) echo '<div class="'.$attributes['className'].'">';
?>
            <?php if(!empty($fields["items_faq"])){
                    
                                            $faq_list = array();
                    
                                            foreach($fields["items_faq"] as $sc_count => $faq_item){
                                                $faq_list[$sc_count] = $faq_item["id"];
                                            }   
                    
                                            
                    
                                            $sub_query = new WP_Query( array( 'post_type' => 'faq', 'post__in' => $faq_list, 'orderby' => 'post__in' ) );
                                            $counter = '0';
                                            
                                            while ( $sub_query->have_posts() ) : $sub_query->the_post();
                                            
                                            echo display_faq_listing_item(get_the_ID());
                                            
                                            endwhile;
                    
                                            
                                            wp_reset_postdata();
                                        }
                                        ?>
<?php   if(isset($attributes['className'])) echo '</div>';
} );
    /* End Custom Block FAQs */

    /* End FAQs Custom Post Type */

    /* End Custom Wordpress Post Types  (only include this once) */ 
}