<?php
use Carbon_Fields\Container;
use Carbon_Fields\Block;
use Carbon_Fields\Field;
/**
 * Functions and definitions
 */

/**
 * Enqueue the style.css file.
 * 
 * @since 1.0.0
 */

add_theme_support( 'post-thumbnails' );

add_theme_support( 'editor-styles' );

add_editor_style( 'editor-style.css' );

remove_filter( 'the_content', 'wpautop' );
add_filter('wpcf7_autop_or_not', '__return_false');

function pbt_styles() {

    wp_enqueue_style(
        'splide-core',
        get_template_directory_uri(). '/css/vendor/splide-core.min.css',
        array(),
        wp_get_theme()->get( 'Version' )
    );
    
    wp_enqueue_script('source-js-jquery', get_template_directory_uri() . "/js/vendor/jquery-3.1.1.min.js", array(), wp_get_theme()->get( 'Version' ), 'true' );
    wp_enqueue_script('new-js-splide', get_template_directory_uri() . '/js/vendor/splide.min.js', array(), wp_get_theme()->get( 'Version' ), 'true' );
    wp_enqueue_script('new-js-pmcms', get_template_directory_uri() . '/js/pmcms.js', array(), wp_get_theme()->get( 'Version' ), 'true' );

}
add_action( 'wp_enqueue_scripts', 'pbt_styles' );

function addTemplateScript($script){
    wp_add_inline_script('source-js-jquery',$script,'after');
}

if ( ! function_exists( 'pbt_setup' ) ) {
    function pbt_setup() {
            add_theme_support( 'wp-block-styles' );
        }
}
add_action( 'after_setup_theme', 'pbt_setup' );

/* Custom Color Scheme */

function pannos_cms_admin_color_scheme() {
  //Get the theme directory
  $theme_dir = get_template_directory_uri();

  //Pannos CMS
  wp_admin_css_color( 'pannos_cms', __( 'Pannos CMS Blue' ),
    $theme_dir . '/admin-themes/pannos_cms.css',
    array( '#00294d', '#005097', '#e7eff2', '#d15e00')
  );
  //Pannos CMS Teal
  wp_admin_css_color( 'pannos_cms_teal', __( 'Pannos CMS Teal' ),
    $theme_dir . '/admin-themes/pannos_cms_teal.css',
    array( '#63ccca', '#00393A', '#badbd8', '#AA3700')
  );
  //Pannos CMS Magenta
  wp_admin_css_color( 'pannos_cms_magenta', __( 'Pannos CMS Magenta' ),
    $theme_dir . '/admin-themes/pannos_cms_magenta.css',
    array( '#f54d80', '#7c576d', '#FFF4F8' , '#2c286e')
  );
  //Pannos CMS Vintage
  wp_admin_css_color( 'pannos_cms_vintage', __( 'Pannos CMS Vintage' ),
    $theme_dir . '/admin-themes/pannos_cms_vintage.css',
    array( '#56514C' , '#D6E1E5', '#F2F0E5', '#6694AD')
  );
}
add_action('admin_init', 'pannos_cms_admin_color_scheme');


/* Shortcodes to replace Pannos shortcuts */

function post_title_shortcode(){
    return get_the_title();
}
add_shortcode('page_name','post_title_shortcode');

add_shortcode( 'year', function() {return date('Y');} );

function breadcrumb_shortcode(){
    // return do_shortcode("[wpseo_breadcrumb]");
    return '<nav id="breadcrumb" class="breadcrumb" aria-label="Breadcrumb">'.do_shortcode("[wpseo_breadcrumb]").'</nav>';
}
add_shortcode('breadcrumb','breadcrumb_shortcode');

function uid_shortcode($atts){

    $atts = shortcode_atts( array(
        'id' => ''
    ), $atts, 'uid_shortcode' );

    if(!empty($atts['id'])){
        return $atts['id'];
    } else { return uniqid(); }

}
add_shortcode('uid','uid_shortcode');

/* Menu Shortcode */

function display_menu_shortcode($atts){

    $atts = shortcode_atts( array(
        'menu' => 'Main Menu',
        'base' => 'Home',
        'show_children' => 1, 
        'show_grandchildren' => 0, 
        'show_siblings' => 0, 
        'expand_to_current' => 0
                        ), $atts, 'display_menu_shortcode' );

    return '<!-- Menu: '.$atts['menu'].' -->'.displayMenu($atts['menu'], $atts['base'], $atts['show_children'], $atts['show_grandchildren'], $atts['show_siblings'], $atts['expand_to_current'],false);
}
add_shortcode('display_menu','display_menu_shortcode');

/** Disable Curly Quotes */
remove_filter('the_content', 'wptexturize');
remove_filter('the_title', 'wptexturize');
remove_filter('the_excerpt', 'wptexturize');

/* Enable Main Menu */

function register_main_menu() {
  register_nav_menu('main-menu',__( 'Main Menu' ));
}
add_action( 'init', 'register_main_menu' );

function register_mobile_menu() {
  register_nav_menu('mobile-menu',__( 'Mobile Menu' ));
}
add_action( 'init', 'register_mobile_menu' );

function displayMenu($menuName, $base, $show_children, $show_grandchildren, $show_siblings, $expand_to_current, $echo=true, $aria_label="", $menu_classes='') {

    $current_page_path = untrailingslashit($_SERVER['REQUEST_URI']);
    // $current_page = get_page_by_path( $current_page_path );

    if(empty($menu_classes)) $menu_classes = 'menu_nav cm_'.sanitize_title($menuName);


    $base_id = false;
    $base_parent_id = 0;


    $subnav_args = array(
        'base_id' => $base_id,
        'show_children' => $show_children, 
        'show_grandchildren' => $show_grandchildren, 
        'show_siblings' => $show_siblings, 
        'expand_to_current' => $expand_to_current
    );
    
    $nav_menu = '';

    $nav_menu .= '<!-- Menu Name = '.$menuName.' -->';

    if(!empty($base) && strtolower($base) != 'home'){

        if ( $base_post = get_page_by_slug( sanitize_title($base), 'nav_menu_item' ) ){
                $base_id = $base_post[0]->ID;
            }
            else if( $base_post = get_post($base)){
                $base_id = $base_post->ID;
            }

        $nav_menu .= '<!-- Base ID = '.$base_id.' because base is '.$base.' -->';
    }
    if($base_id){

        $menu_array = wp_get_nav_menu_items( $menuName );
        // $nav_menu .= '<!-- Base ID = '.$base_id.' '.print_r($base_post,true).' -->';

        if($show_siblings){ 
            if(empty($aria_label)) $aria_label = ucfirst($menuName).' Menu';
            $nav_menu .= '<nav class="'.$menu_classes.'" aria-label="'.$aria_label.'">'; 
        }

        $nav_menu_items = array();

        foreach ($menu_array as $menu_order => $menu_item) {            
            if($menu_item->ID == $base_id){
                $nav_menu_items[$menu_order] = '';
                if(!$show_siblings){ 
                    if(empty($aria_label)) $aria_label = $menu_item->post_title.' Menu';
                    $nav_menu_items[$menu_order] .= '<nav class="'.$menu_classes.'" aria-label="'.$aria_label.'">';
                } else {                    
                    $base_parent_id = $menu_item->menu_item_parent;
                    $nav_menu .= '<!-- Base Parent ID = '.$base_parent_id.' -->';
                }
                if($show_children || $show_grandchildren || $expand_to_current){ $nav_menu_items[$menu_order] .= '<ul>'; }
                if($expand_to_current) { 
                    $nav_menu_items[$menu_order] .= "<li class='item_spot_".$menu_order." expanded ".((untrailingslashit($menu_item->url) == $current_page_path) ? "current is_parent" : "")."'>
                    <a target='".$menu_item->target."' class='".implode(' ', $menu_item->classes)."' data-nav-item-id='".$menu_item->post_name."' data-nav-item-parent-id='".$menu_item->menu_item_parent."' href='".$menu_item->url."'>".$menu_item->post_title."</a>";
                    if($show_children || $show_grandchildren){ $nav_menu_items[$menu_order] .= '<ul>'; }
                }
                foreach ($menu_array as $inner_menu_order => $inner_menu_item) {
                    if($inner_menu_item->menu_item_parent == $base_id){
                        if($show_children){ 
                            $nav_menu_items[$menu_order] .= "<li class='item_spot_".$inner_menu_order." expanded ".((untrailingslashit($inner_menu_item->url) == $current_page_path) ? "current is_parent" : " not_current")."'>
                            <!-- Base Parent ID = ".untrailingslashit($inner_menu_item->url)." vs ".$current_page_path." -->";
                            $nav_menu_items[$menu_order] .= "<a target='".$inner_menu_item->target."' class='".implode(' ', $inner_menu_item->classes)."' data-nav-item-id='".$inner_menu_item->post_name."' data-nav-item-parent-id='".$inner_menu_item->menu_item_parent."' href='".$inner_menu_item->url."'>".$inner_menu_item->post_title."</a>";
                        }
                        if($show_grandchildren){
                            if($show_children){ $nav_menu_items[$menu_order] .= "<ul>"; }
                            $gc_base_id = $inner_menu_item->ID;
                            foreach ($menu_array as $gc_menu_order => $gc_menu_item) {
                                if($gc_menu_item->menu_item_parent == $gc_base_id){
                                    $nav_menu_items[$menu_order] .= "<li class='item_spot_".$gc_menu_order." expanded ".((untrailingslashit($gc_menu_item->url) == $current_page_path) ? "current is_parent" : "")."'>
                                        <a target='".$gc_menu_item->target."' class='".implode(' ', $gc_menu_item->classes)."' data-nav-item-id='".$gc_menu_item->post_name."' data-nav-item-parent-id='".$gc_menu_item->menu_item_parent."' href='".$gc_menu_item->url."'>".$gc_menu_item->post_title."</a>";
                                    $nav_menu_items[$menu_order] .= "</li>";
                                }
                            }
                            if($show_children){ $nav_menu_items[$menu_order] .= "</ul>"; }
                        }

                        if($show_children){ $nav_menu_items[$menu_order] .= "</li>"; }
                    }
                }
                if($expand_to_current){ 
                    if($show_children || $show_grandchildren){ $nav_menu_items[$menu_order] .= '</ul>'; }
                    $nav_menu_items[$menu_order] .= "</li>";
                }
                if($show_children || $show_grandchildren || $expand_to_current){ $nav_menu_items[$menu_order] .= '</ul>'; }
                if(!$show_siblings){ $nav_menu_items[$menu_order] .= '</nav>'; }
            }
        }

        if($show_siblings){
            foreach ($menu_array as $sibling_order => $sibling_item) {                
                if($sibling_item->menu_item_parent == $base_parent_id && $sibling_item->ID != $base_id){
                    $nav_menu_items[$sibling_order] = '';
                    $nav_menu_items[$sibling_order] .= '<!-- Base Parent ID = '.$base_parent_id.' vs '.$sibling_item->menu_item_parent.' -->';
                    if($show_children || $show_grandchildren || $expand_to_current){ $nav_menu_items[$sibling_order] .= '<ul>'; }
                    $nav_menu_items[$sibling_order] .= "<li class='item_spot_".$sibling_order."'>
                        <a target='".$sibling_item->target."' class='".implode(' ', $sibling_item->classes)."' data-nav-item-id='".$sibling_item->post_name."' href='".$sibling_item->url."'>".$sibling_item->post_title."</a>";
                    $nav_menu_items[$sibling_order] .= "</li>";
                    if($show_children || $show_grandchildren || $expand_to_current){ $nav_menu_items[$sibling_order] .= '</ul>'; }
                }
            }
        }

        $nav_menu .= implode('
            ', $nav_menu_items);
    } else {
        if(empty($aria_label)) $aria_label = $menuName.' menu';
        $nav_menu .= wp_nav_menu( array( 'menu' => $menuName, 'container' => 'nav', 'container_class' => $menu_classes, 'container_aria_label' => $aria_label, 'menu_class' => '', 'echo' => false) );
    }    
    
    if($echo){
        echo $nav_menu;
        return false;
    } else {
        return $nav_menu;
    }
}

function get_page_by_slug($the_slug,$post_type){
    $args = array(
        'name'           => $the_slug,
        'post_type'      => $post_type,
        'post_status'    => 'publish',
        'posts_per_page' => 1
    );
    return get_posts( $args );
}

/* Return Uploads Folder Location */

function pm_uploads_folder_url() {
    $upload_dir = wp_get_upload_dir();
    return $upload_dir['baseurl'];
}

/* Basic Custom Search based on Pannos CMS search */

function pm_query_vars( $qvars ) {
$qvars[] = 'search_key';
return $qvars;
}
add_filter( 'query_vars', 'pm_query_vars' );

function search_results_shortcode($atts){

    $atts = shortcode_atts( array(
        'post_type' => 'page'
    ), $atts, 'search_results_shortcode' );

    $post_type_array = explode(',', $atts['post_type']);
    foreach($post_type_array as $i => $post_type){
        $post_type_array[$i] = trim($post_type);
    }

    $search_key = htmlentities(get_query_var('search_key'), ENT_QUOTES);
    $args = array(
                's' =>$search_key,
                'post_type' => $post_type_array
            );
    // The Query
    $the_search_query = new WP_Query( $args );

    ob_start();

    echo '<div class="product-page-item">
    <div class="product-item-content clearfix">';

    if ( !empty($search_key) && $the_search_query->have_posts() ) {
            if(is_numeric($search_key)){
                _e('<p class="search_results_amount">Your search has produced <strong>'.$the_search_query->found_posts.'</strong> results</p>');
            } else {
                _e('<p class="search_results_amount">Your search for <strong>"'.$search_key.'"</strong> has produced <strong>'.$the_search_query->found_posts.'</strong> results</p>');
            }
        _e('
            <ul id="search_results">');
        $alt_row = 1;
        while ( $the_search_query->have_posts() ) {
            $the_search_query->the_post();

            $meta_desc_output = '';
            $meta_desc = get_post_meta(get_the_ID(), '_yoast_wpseo_metadesc', true);
            if(!empty($meta_desc)){
                $meta_desc_output = '<p>'.$meta_desc.'</p>';
            }

             ?>
                <li class="<?php echo ($alt_row ? 'alt_row' : ''); ?>">
                    <h4><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h4>
                    <?php echo $meta_desc_output; ?>
                    <p class="search_page_path"><?php echo wp_make_link_relative(get_permalink()); ?></p>
                </li>
             <?php
             $alt_row = !$alt_row;
        }
        _e('</ul>');
    } else{
        _e('<p class="search_results_amount">Your search for <strong>"'.$search_key.'"</strong> has produced <strong>0</strong> results</p>');
    }

    $output = ob_get_clean(); 

    return $output;

}
if ( !is_admin() ) add_shortcode('pm_search_results','search_results_shortcode');

/* Base Custom Post Types */

require_once('custom-post-types/custom-post-types-standard.php');

/* Base Custom Blocks */

add_action( 'carbon_fields_register_fields', 'crb_custom_blocks_base' );
function crb_custom_blocks_base() {

    /* Menu Shortcode Generator Block */

/* Custom Block: Menu */
Block::make( __( 'Menu' ) )
    ->add_fields( array(
    Field::make( 'html', 'block_title' )->set_html( '<h2>Menu</h2>' ),
    Field::make( 'association', 'menu', __( 'Menu' ) )->set_types(
        array( 
            array(
                'type' => 'term',
                'taxonomy' => 'nav_menu',
            )
        ) 
    )->set_max( 1 ),
    Field::make( 'association', 'base', __( 'Menu Base Page' ) )->set_types(
        array( 
            array(
                'type' => 'post',
                'post_type' => 'nav_menu_item',
            )
        ) 
    )->set_max( 1 ),
    Field::make( 'checkbox', 'showchildren', __( 'Show Children' ) )->set_option_value( '1' )->set_width( 24 ),
    Field::make( 'checkbox', 'showgrandchildren', __( 'Show Grandchildren' ) )->set_option_value( '1' )->set_width( 24 ),
    Field::make( 'checkbox', 'showsiblings', __( 'Show Siblings' ) )->set_option_value( '1' )->set_width( 24 ),
    Field::make( 'checkbox', 'expandcurrent', __( 'Expand to Current' ) )->set_option_value( '1' )->set_width( 24 )
    ) )->set_category( 'pannos-blocks', __( 'Pannos Template Blocks' ), 'schedule' )
->set_icon( 'menu' )
->set_render_callback( function ($fields, $attributes, $inner_blocks) {

        $block_uid = uniqid();

        // echo 'Menu: <pre>'.print_r($fields["menu"],true).'</pre>';

        // echo 'Base: <pre>'.print_r($fields["base"],true).'</pre>';
     
            if(isset($attributes['className'])) echo '<div class="'.$attributes['className'].'">';
?>
            [display_menu 
            menu='<?php if(!empty($fields["menu"]) && !empty($fields["menu"][0]['id'])){ echo $fields["menu"][0]['id']; } ?>' 
            base='<?php if(!empty($fields["base"]) && !empty($fields["base"][0]['id'])){ echo $fields["base"][0]['id']; } ?>' 
            show_children='<?php if(!empty($fields["showchildren"])){ echo $fields["showchildren"]; } else { echo '0'; } ?>' 
            show_grandchildren='<?php if(!empty($fields["showgrandchildren"])){ echo $fields["showgrandchildren"]; } else { echo '0'; } ?>' 
            show_siblings='<?php if(!empty($fields["showsiblings"])){ echo $fields["showsiblings"]; } else { echo '0'; } ?>' 
            expand_to_current='<?php if(!empty($fields["expandcurrent"])){ echo $fields["expandcurrent"]; } else { echo '0'; } ?>']
<?php   if(isset($attributes['className'])) echo '</div>';
} );
    
    /* End Custom Block Menu */

    /* End custom blocks */
}

/* Custom Settings Fields */

add_action( 'carbon_fields_register_fields', 'crb_attach_pw_theme_options' );
function crb_attach_pw_theme_options() {
    Container::make( 'theme_options', __( 'Pannos Theme Options' ) )
        ->add_fields( array(
            Field::make( 'textarea', 'crb_head_analytics', 'Header Analytics' )->set_rows( 15 ),
            Field::make( 'textarea', 'crb_body_analytics', 'Body Analytics' )->set_rows( 15 ),
            Field::make( 'text', 'pw_allowed_preview_types', __( 'Post Types Allowing Preview Links' ) )
        ) );

    /* Add Settings for Preview Links */

    date_default_timezone_set('America/New_York');
    $dateTime = new DateTime();
    $begin_date = $dateTime->format('ddmmyyyy');

    $post_types_list = carbon_get_theme_option( 'pw_allowed_preview_types' );
    if(!empty($post_types_list)){
        $post_types_array = explode(',', $post_types_list);

        Container::make( 'post_meta', 'Preview Link' )
        ->where( 'post_type', 'IN', $post_types_array  )
        ->set_context( 'side' )->set_priority( 'low' )
        ->add_fields( array(
            Field::make( 'checkbox', 'pw_allow_preview', 'Allow Draft Preview' )
            ->set_attribute( 'data-disabled', 'conditional' ),
            Field::make( 'hidden', 'pm_preview_key', __( '' ) )->set_default_value( $begin_date ),
            Field::make( 'html', 'pm_preview_link_status' )
            ->set_html( 
                function(){
                    $post_id = $_GET['post'] ?? $_POST['id'] ?? $_POST['post_ID'];                     
                
                    $current_status = get_post_status($post_id);
                    if($current_status != 'draft'){
                        return '<style>#carbon_fields_container_preview_link { display: none; } </style>'; 
                    }
                
                } ),
            Field::make( 'html', 'pm_preview_link' )
            ->set_html( 
                function(){
                    $post_id = $_GET['post'] ?? $_POST['id'] ?? $_POST['post_ID'];                     
                    if(!empty($post_id) && carbon_get_post_meta( $post_id, 'pw_allow_preview' ) ){
                        $current_status = get_post_status($post_id);
                        if($current_status == 'draft'){
                            $dateTime = new DateTime();
                            $begin_date = $dateTime->format('ddmmyyyy');
                            $preview_key = $begin_date;                            
                            if(!empty(carbon_get_post_meta( $current_id, 'pm_preview_key' ))) $preview_key = carbon_get_post_meta( $current_id, 'pm_preview_key' );
                            $preview_url_key = hash('adler32', $preview_key.$post_id);

                            $link_url_format = '?page_id=';
                            $current_post_type = get_post_type($post_id);
                            if($current_post_type != 'page') $link_url_format = '?post_type='.$current_post_type.'&p=';

                            return '
                            <a class="share-link" href="/'.$link_url_format.$post_id.'&preview=true&aprpro='.$preview_url_key.'" target="_blank"><span>Share</span><svg xmlns="http://www.w3.org/2000/svg" fill="#d15e00" viewBox="0 0 24 24" width="24" height="24" class="components-menu-items__item-icon has-icon-right" aria-hidden="true" focusable="false"><path d="M19.5 4.5h-7V6h4.44l-5.97 5.97 1.06 1.06L18 7.06v4.44h1.5v-7Zm-13 1a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2v-3H17v3a.5.5 0 0 1-.5.5h-10a.5.5 0 0 1-.5-.5v-10a.5.5 0 0 1 .5-.5h3V5.5h-3Z"></path></svg></a>
                            <style>.container-carbon_fields_container_preview_link .share-link { display: inline-flex; font-size: 1.2em; font-weight: 500; align-items: center; } .cf-container .cf-field+.cf-field.cf-hidden { border: 0; padding: 2.5px; } .cf-container .cf-field+.cf-field.cf-hidden { border: 0; padding: 2.5px; } .cf-container .cf-field+.cf-field.cf-html { padding: 3px 12px; } .cf-container .cf-field+.cf-field { border-top: 0; }</style>
                            ';
                        } else {
                            return '<style>#carbon_fields_container_preview_link { display: none; } </style>'; 
                            // return '<p>Preview links can\'t be used directly for published posts. You can use the "Rewrite & Republish" feature to create a temporary draft copy, or the "Switch to Draft" option to unpublish the post itself.</p>';
                        }
                    } else {
                        if(!empty($post_id) && $current_status = get_post_status($post_id) == 'draft'){
                            return '
                            <p>Save draft and refresh to generate shareable preview link.</a><style>.cf-container .cf-field+.cf-field.cf-hidden { border: 0; padding: 2.5px; } .cf-container .cf-field+.cf-field.cf-html { padding: 3px 12px; } .cf-container .cf-field+.cf-field { border-top: 0; }</style>
                            ';
                        } else {                            
                            return '<p>Preview links can\'t be used directly for published posts. You can use the <strong>"Rewrite & Republish"</strong> feature to create a temporary draft copy, or the <strong>"Switch to Draft"</strong> option to unpublish the post itself.</p>';
                        }
                    }
                } )
            ->set_conditional_logic( array(
                array(
                    'field' => 'pw_allow_preview',
                    'value' => true,
                )
            ) ),
            Field::make( 'date', 'pw_preview_end', __( 'Preview Expiration' ) )
            ->set_storage_format( 'Ymd' )
            ->set_conditional_logic( array(
                array(
                    'field' => 'pw_allow_preview',
                    'value' => true,
                )
            ) ),
            ));
    }
}

/* Add Analytics from Pannos Theme Settings Screen to Head and Footer */

if(!function_exists('pm_analytics_head')){
    function pm_analytics_head() {
        echo carbon_get_theme_option( 'crb_head_analytics' );
    }
    add_action('wp_head', 'pm_analytics_head');
}

if(!function_exists('pm_analytics_body')){
    function pm_analytics_body() {
        echo carbon_get_theme_option( 'crb_body_analytics' );
    }
    add_action( 'wp_footer', 'pm_analytics_body', 100 );
}

/* Custom Post Type Form Fields */

add_action( 'wpcf7_init', 'custom_add_form_tag_customlist' );

function custom_add_form_tag_customlist() {
    wpcf7_add_form_tag( array( 'customlist', 'customlist*' ), 
'custom_customlist_form_tag_handler', array( 'name-attr' => true ) );
}

function custom_customlist_form_tag_handler( $tag ) {

    $tag = new WPCF7_FormTag( $tag );

    if ( empty( $tag->name ) ) {
        return '<!-- NO NAME FOUND -->';
    }

    $custom_post_type = '';

    foreach ( $tag->values as $val ) {
        $custom_post_type = esc_html( $val );
    }

    $customlist = '';

    foreach ( $tag->options as $opt ) {

        if($opt == 'include_blank') $customlist = '<option value="">-- Select --</option>';
        
    }


    if ( empty( $custom_post_type ) ) {
        return '<!-- NO POST TYPE FOUND (should be in quotes in the tag, to appear under "values" here: '.print_r($tag,true).' -->';
    }

    $query = new WP_Query(array(
        'post_type' => $custom_post_type,
        'post_status' => 'publish',
        'posts_per_page' => -1,
        'orderby'       => 'menu_order',
        'order'         => 'ASC',
    ));

    while ($query->have_posts()) {
        $query->the_post();
        $post_title = get_the_title();
        $customlist .= sprintf( '<option value="%1$s">%2$s</option>', 
esc_html( $post_title ), esc_html( $post_title ) );
    }

    wp_reset_query();


    if ( empty( $customlist ) ) {
        return '<!-- NO POSTS FOUND for post type "'.$custom_post_type.'" -->';
    }

    $customlist = sprintf(
        '<select name="%1$s" id="%2$s">%3$s</select>', $tag->name,
    $tag->name . '-options',
        $customlist );

    return $customlist;
}

if(!function_exists('pm_link_list')){
    function pm_link_list(){
        $args = array(
        'post_type' => array( 'page', 'nav_menu_item', 'links'),
        'posts_per_page' => -1,
        'update_post_term_cache' => false
        );
        $the_link_array = array();
        $the_link_array[''] = '';
        $the_link_query = new WP_Query( $args );
        $links = $the_link_query->posts;
        foreach($links as $link_info){

            $URL = '';
            $link_title = $link_info->post_title;

            if($link_info->post_type == 'nav_menu_item'){
                $URL = get_post_meta($link_info->ID, '_menu_item_url', true);
                if(substr( $URL, 0, 1 ) === "#"){
                    $menu_parent = get_post_meta($link_info->ID, '_menu_item_menu_item_parent');
                    if(!empty($menu_parent[0])){
                        $URL = get_post_meta($menu_parent[0], '_menu_item_url', true).$URL;
                    }
                }
            } else if($link_info->post_type == 'links'){
                $URL = carbon_get_post_meta( $link_info->ID, 'url' );
                $link_title = carbon_get_post_meta( $link_info->ID, 'title' );
            } else {
                $URL = get_permalink($link_info);
            }

            // echo '<pre>LINK INFO: 
            // title: '.$link_info->post_title.' 
            // URL: '.$URL.'
            // </pre>';
            $final_url = preg_replace('!http(s)?://' . $_SERVER['SERVER_NAME'] . '/!', '/', $URL);
            $the_link_array[$final_url] = $link_title.' ('.$final_url.')';
        }
        asort($the_link_array);        
        $the_link_array[''] = ' Select a Page';
        return $the_link_array;
    }
    add_shortcode('link_list','pm_link_list');
}

/* Enable normal WP shortcodes within Contact Form 7 output */

add_filter( 'wpcf7_form_elements', 'do_shortcode' );


/* Add Query Variable for Preview Links */

add_filter('query_vars', 'preview_link_query_var');

function preview_link_query_var($query_vars)
{
    $query_vars[] = 'aprpro'; // Change it to your desired name
    return $query_vars;
}

add_action('pre_get_posts', 'allow_draft_preview_link');
 
function allow_draft_preview_link($query) {
    date_default_timezone_set('America/New_York');
    /* Check for preview flag in URL, and a non-logged in user status */
    $preview_url_key = get_query_var( 'aprpro', false);
    if ($preview_url_key && !is_user_logged_in()) {
        /* Check if this post query is the main one */
        if ($query->is_main_query()) {
            /* Check the current post ID for the relevant meta values */
            $current_id = get_query_var( 'page_id');
            if(empty($current_id) && !empty(get_query_var( 'post_type')) ) $current_id = get_query_var( 'p');
            echo '<!-- preview_url: '. $preview_url_key.' -->';
            echo '<!-- current_id: '. $current_id.' -->';
            /* Check whether preview URL is enabled for this post */
            if(carbon_get_post_meta( $current_id, 'pw_allow_preview' )){
                /* Get the key value saved for this post, and run it through the same hash algorithm as used in the link, then compare */
                $preview_key = carbon_get_post_meta( $current_id, 'pm_preview_key' );    
                $compare_key = hash('adler32', $preview_key.$current_id);
                    echo '<!-- preview_key: '. $preview_key.' -->';
                    echo '<!-- compare_key: '. $compare_key.' -->';
                if($compare_key == $preview_url_key){
                    /* Check the expiration date and current date */

                    $pw_preview_end = carbon_get_post_meta( $current_id, 'pw_preview_end' );
                    $dateTime = new DateTime();
                    $current_date = $dateTime->format('Ymd');
                    echo '<!-- today: '. $current_date.' -->';
                    echo '<!-- expire: '. $pw_preview_end.' -->';
                    /* If the expiration date is unset, or in the future, show the draft */
                    if(empty($pw_preview_end) || $current_date <= $pw_preview_end) $query->set('post_status', array('publish', 'draft'));
                }
            }
        }
    }
}



add_action('enqueue_block_editor_assets', function () {
    wp_enqueue_script(
        'fields-script',
        get_template_directory_uri() . '/js/fields.js',
        array('wp-blocks', 'wp-dom', 'jquery', 'carbon-fields-vendor')
    );
});